@@ -1,6 +1,6 @@
 /* 
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2010 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2012 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -50,6 +50,7 @@
 #define RFC3361	(1 << 10)
 #define RFC3397	(1 << 11)
 #define RFC3442 (1 << 12)
+#define RFC5969 (1 << 13)
 
 #define IPV4R	IPV4 | REQUEST
 
@@ -159,6 +160,8 @@ static const struct dhcp_opt const dhcp_opts[] = {
 	{ 114,	STRING,		"default_url" },
 	{ 118,	IPV4,		"subnet_selection" },
 	{ 119,	STRING | RFC3397,	"domain_search" },
+	{ 120,	STRING | RFC3361,	"sip_server" },
+	{ 212,  RFC5969,	"sixrd" },
 	{ 0, 0, NULL }
 };
 
@@ -266,18 +269,17 @@ valid_length(uint8_t option, int dl, int *type)
 			*type = opt->type;
 
 		if (opt->type == 0 ||
-		    opt->type & STRING ||
-		    opt->type & RFC3442)
+		    opt->type & (STRING | RFC3442 | RFC5969))
 			return 0;
 
 		sz = 0;
-		if (opt->type & UINT32 || opt->type & IPV4)
+		if (opt->type & (UINT32 | IPV4))
 			sz = sizeof(uint32_t);
 		if (opt->type & UINT16)
 			sz = sizeof(uint16_t);
 		if (opt->type & UINT8)
 			sz = sizeof(uint8_t);
-		if (opt->type & IPV4 || opt->type & ARRAY)
+		if (opt->type & (IPV4 | ARRAY))
 			return dl % sz;
 		return (dl == sz ? 0 : -1);
 	}
@@ -424,7 +426,7 @@ get_option_uint8(uint8_t *i, const struct dhcp_message *dhcp, uint8_t option)
  * separated string. Returns length of string (including
  * terminating zero) or zero on error. out may be NULL
  * to just determine output length. */
-static ssize_t
+ssize_t
 decode_rfc3397(char *out, ssize_t len, int pl, const uint8_t *p)
 {
 	const uint8_t *r, *q = p;
@@ -620,11 +622,11 @@ decode_rfc3361(int dl, const uint8_t *data)
 		addr.s_addr = INADDR_BROADCAST;
 		l = ((dl / sizeof(addr.s_addr)) * ((4 * 4) + 1)) + 1;
 		sip = p = xmalloc(l);
-		while (l != 0) {
+		while (dl != 0) {
 			memcpy(&addr.s_addr, data, sizeof(addr.s_addr));
 			data += sizeof(addr.s_addr);
 			p += snprintf(p, l - (p - sip), "%s ", inet_ntoa(addr));
-			l -= sizeof(addr.s_addr);
+			dl -= sizeof(addr.s_addr);
 		}
 		*--p = '\0';
 		break;
@@ -636,6 +638,74 @@ decode_rfc3361(int dl, const uint8_t *data)
 	return sip;
 }
 
+/* Decode an RFC5969 6rd order option into a space
+ * separated string. Returns length of string (including
+ * terminating zero) or zero on error. */
+static ssize_t
+decode_rfc5969(char *out, ssize_t len, int pl, const uint8_t *p)
+{
+	uint8_t ipv4masklen, ipv6prefixlen;
+	uint8_t ipv6prefix[16];
+	uint8_t br[4];
+	int i;
+	ssize_t b, bytes = 0;
+
+	if (pl < 22) {
+		errno = EINVAL;
+		return 0;
+	}
+	
+	ipv4masklen = *p++;
+	pl--;
+	ipv6prefixlen = *p++;
+	pl--;
+	
+	for (i = 0; i < 16; i++) {
+		ipv6prefix[i] = *p++;
+		pl--;
+	}
+	if (out) {
+		b= snprintf(out, len,
+		    "%d %d "
+		    "%02x%02x:%02x%02x:"
+		    "%02x%02x:%02x%02x:"
+		    "%02x%02x:%02x%02x:"
+		    "%02x%02x:%02x%02x",
+		    ipv4masklen, ipv6prefixlen,
+		    ipv6prefix[0], ipv6prefix[1], ipv6prefix[2], ipv6prefix[3],
+		    ipv6prefix[4], ipv6prefix[5], ipv6prefix[6], ipv6prefix[7],
+		    ipv6prefix[8], ipv6prefix[9], ipv6prefix[10],ipv6prefix[11],
+		    ipv6prefix[12],ipv6prefix[13],ipv6prefix[14], ipv6prefix[15]
+		);
+		    
+		len -= b;
+		out += b;
+		bytes += b;
+	} else {
+		bytes += 16 * 2 + 8 + 2 + 1 + 2;
+	}
+
+	while (pl >= 4) {
+		br[0] = *p++;
+		br[1] = *p++;
+		br[2] = *p++;
+		br[3] = *p++;
+		pl -= 4;
+		
+		if (out) {
+			b= snprintf(out, len, " %d.%d.%d.%d",
+			    br[0], br[1], br[2], br[3]);
+			len -= b;
+			out += b;
+			bytes += b;
+		} else {
+			bytes += (4 * 4);
+		}
+	}
+	
+	return bytes;
+}
+
 char *
 get_option_string(const struct dhcp_message *dhcp, uint8_t option)
 {
@@ -892,7 +962,6 @@ make_message(struct dhcp_message **message,
 
 	if (lease->addr.s_addr && lease->cookie == htonl(MAGIC_COOKIE)) {
 		if (type == DHCP_DECLINE ||
-		    type == DHCP_DISCOVER ||
 		    (type == DHCP_REQUEST &&
 			lease->addr.s_addr != iface->addr.s_addr))
 		{
@@ -915,6 +984,9 @@ make_message(struct dhcp_message **message,
 		p += len;
 	}
 
+	if (type == DHCP_DISCOVER && ifo->options & DHCPCD_REQUEST)
+		PUTADDR(DHO_IPADDRESS, ifo->req_addr);
+
 	if (type == DHCP_DISCOVER ||
 	    type == DHCP_INFORM ||
 	    type == DHCP_REQUEST)
@@ -1053,10 +1125,8 @@ write_lease(const struct interface *iface, const struct dhcp_message *dhcp)
 	    iface->name, iface->leasefile);
 
 	fd = open(iface->leasefile, O_WRONLY | O_CREAT | O_TRUNC, 0444);
-	if (fd == -1) {
-		syslog(LOG_ERR, "%s: open: %m", iface->name);
+	if (fd == -1)
 		return -1;
-	}
 
 	/* Only write as much as we need */
 	while (p < e) {
@@ -1141,7 +1211,9 @@ print_string(char *s, ssize_t len, int dl, const uint8_t *data)
 		case '\'': /* FALLTHROUGH */
 		case '$':  /* FALLTHROUGH */
 		case '`':  /* FALLTHROUGH */
- 		case '\\':
+ 		case '\\': /* FALLTHROUGH */
+		case '|':  /* FALLTHROUGH */
+		case '&':
 			if (s) {
 				if (len < 3) {
 					errno = ENOBUFS;
@@ -1191,9 +1263,21 @@ print_option(char *s, ssize_t len, int type, int dl, const uint8_t *data)
 		return l;
 	}
 
+	if (type & RFC3361) {
+		if ((tmp = decode_rfc3361(dl, data)) == NULL)
+			return -1;
+		l = strlen(tmp);
+		l = print_string(s, len, l - 1, (uint8_t *)tmp);
+		free(tmp);
+		return l;
+	}
+
 	if (type & RFC3442)
 		return decode_rfc3442(s, len, dl, data);
 
+	if (type & RFC5969)
+		return decode_rfc5969(s, len, dl, data);
+
 	if (type & STRING) {
 		/* Some DHCP servers return NULL strings */
 		if (*data == '\0')
@@ -1271,16 +1355,6 @@ print_option(char *s, ssize_t len, int type, int dl, const uint8_t *data)
 	return bytes;
 }
 
-static void
-setvar(char ***e, const char *prefix, const char *var, const char *value)
-{
-	size_t len = strlen(prefix) + strlen(var) + strlen(value) + 4;
-
-	**e = xmalloc(len);
-	snprintf(**e, len, "%s_%s=%s", prefix, var, value);
-	(*e)++;
-}
-
 ssize_t
 configure_env(char **env, const char *prefix, const struct dhcp_message *dhcp,
     const struct if_options *ifo)
