@@ -1,6 +1,6 @@
 /* 
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2010 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2012 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -25,7 +25,7 @@
  * SUCH DAMAGE.
  */
 
-const char copyright[] = "Copyright (c) 2006-2010 Roy Marples";
+const char copyright[] = "Copyright (c) 2006-2012 Roy Marples";
 
 #include <sys/file.h>
 #include <sys/socket.h>
@@ -67,7 +67,9 @@ const char copyright[] = "Copyright (c) 2006-2010 Roy Marples";
 #include "if-options.h"
 #include "if-pref.h"
 #include "ipv4ll.h"
+#include "ipv6rs.h"
 #include "net.h"
+#include "platform.h"
 #include "signals.h"
 
 /* We should define a maximum for the NAK exponential backoff */ 
@@ -93,7 +95,7 @@ static char **ifv;
 static int ifc;
 static char *cffile;
 static char *pidfile;
-static int linkfd = -1;
+static int linkfd = -1, ipv6rsfd = -1;
 
 struct dhcp_op {
 	uint8_t value;
@@ -208,13 +210,13 @@ handle_exit_timeout(_unused void *arg)
 			exit(EXIT_FAILURE);
 	}
 	options &= ~DHCPCD_TIMEOUT_IPV4LL;
-	timeout = (PROBE_NUM * PROBE_MAX) + PROBE_WAIT + 1;
+	timeout = (PROBE_NUM * PROBE_MAX) + (PROBE_WAIT * 2);
 	syslog(LOG_WARNING, "allowing %d seconds for IPv4LL timeout", timeout);
 	add_timeout_sec(timeout, handle_exit_timeout, NULL);
 }
 
 void
-drop_config(struct interface *iface, const char *reason)
+drop_dhcp(struct interface *iface, const char *reason)
 {
 	free(iface->state->old);
 	iface->state->old = iface->state->new;
@@ -243,8 +245,13 @@ stop_interface(struct interface *iface)
 	struct interface *ifp, *ifl = NULL;
 
 	syslog(LOG_INFO, "%s: removing interface", iface->name);
+	if (iface->ras) {
+		ipv6rs_free(iface);
+		iface->ras = NULL;
+		run_script_reason(iface, "ROUTERADVERT");
+	}
 	if (strcmp(iface->state->reason, "RELEASE") != 0)
-		drop_config(iface, "STOP");
+		drop_dhcp(iface, "STOP");
 	close_sockets(iface);
 	delete_timeout(NULL, iface);
 	for (ifp = ifaces; ifp; ifp = ifp->next) {
@@ -348,7 +355,7 @@ send_message(struct interface *iface, int type,
 		if (r == -1) {
 			syslog(LOG_ERR, "%s: send_raw_packet: %m", iface->name);
 			if (!(options & DHCPCD_TEST))
-				drop_config(iface, "FAIL");
+				drop_dhcp(iface, "FAIL");
 			close_sockets(iface);
 			delete_timeout(NULL, iface);
 			callback = NULL;
@@ -407,7 +414,7 @@ start_expire(void *arg)
 
 	syslog(LOG_ERR, "%s: lease expired", iface->name);
 	delete_timeout(NULL, iface);
-	drop_config(iface, "EXPIRE");
+	drop_dhcp(iface, "EXPIRE");
 	unlink(iface->leasefile);
 	if (iface->carrier != LINK_DOWN)
 		start_interface(iface);
@@ -504,7 +511,7 @@ handle_dhcp(struct interface *iface, struct dhcp_message **dhcpp, const struct i
 		/* We should restart on a NAK */
 		log_dhcp(LOG_WARNING, "NAK:", iface, dhcp, from);
 		if (!(options & DHCPCD_TEST)) {
-			drop_config(iface, "NAK");
+			drop_dhcp(iface, "NAK");
 			unlink(iface->leasefile);
 		}
 		close_sockets(iface);
@@ -528,7 +535,17 @@ handle_dhcp(struct interface *iface, struct dhcp_message **dhcpp, const struct i
 			log_dhcp(LOG_WARNING, "reject DHCP", iface, dhcp, from);
 			return;
 		}
-	}		
+	}
+
+	/* Ensure that the address offered is valid */
+	if ((type == 0 || type == DHCP_OFFER || type == DHCP_ACK) &&
+	    (dhcp->ciaddr == INADDR_ANY || dhcp->ciaddr == INADDR_BROADCAST) &&
+	    (dhcp->yiaddr == INADDR_ANY || dhcp->yiaddr == INADDR_BROADCAST))
+	{
+		log_dhcp(LOG_WARNING, "reject invalid address",
+		    iface, dhcp, from);
+		return;
+	}
 
 	/* No NAK, so reset the backoff */
 	state->nakoff = 1;
@@ -631,7 +648,7 @@ handle_dhcp_packet(void *arg)
 	const uint8_t *pp;
 	ssize_t bytes;
 	struct in_addr from;
-	int i;
+	int i, partialcsum = 0;
 
 	/* We loop through until our buffer is empty.
 	 * The benefit is that if we get >1 DHCP packet in our buffer and
@@ -639,10 +656,10 @@ handle_dhcp_packet(void *arg)
 	packet = xmalloc(udp_dhcp_len);
 	for(;;) {
 		bytes = get_raw_packet(iface, ETHERTYPE_IP,
-		    packet, udp_dhcp_len);
+		    packet, udp_dhcp_len, &partialcsum);
 		if (bytes == 0 || bytes == -1)
 			break;
-		if (valid_udp_packet(packet, bytes, &from) == -1) {
+		if (valid_udp_packet(packet, bytes, &from, partialcsum) == -1) {
 			syslog(LOG_ERR, "%s: invalid UDP packet from %s",
 			    iface->name, inet_ntoa(from));
 			continue;
@@ -724,7 +741,7 @@ send_release(struct interface *iface)
 		ts.tv_sec = RELEASE_DELAY_S;
 		ts.tv_nsec = RELEASE_DELAY_NS;
 		nanosleep(&ts, NULL);
-		drop_config(iface, "RELEASE");
+		drop_dhcp(iface, "RELEASE");
 	}
 	unlink(iface->leasefile);
 }
@@ -751,6 +768,8 @@ configure_interface1(struct interface *iface)
 	if (iface->flags & IFF_NOARP ||
 	    ifo->options & (DHCPCD_INFORM | DHCPCD_STATIC))
 		ifo->options &= ~(DHCPCD_ARP | DHCPCD_IPV4LL);
+	if (!(iface->flags & (IFF_POINTOPOINT | IFF_LOOPBACK | IFF_MULTICAST)))
+		ifo->options &= ~DHCPCD_IPV6RS;
 	if (ifo->options & DHCPCD_LINK && carrier_status(iface) == -1)
 		ifo->options &= ~DHCPCD_LINK;
 	
@@ -809,17 +828,23 @@ configure_interface1(struct interface *iface)
 	if (ifo->options & DHCPCD_CLIENTID)
 		syslog(LOG_DEBUG, "%s: using ClientID %s", iface->name,
 		    hwaddr_ntoa(iface->clientid + 1, *iface->clientid));
+	else
+		syslog(LOG_DEBUG, "%s: using hwaddr %s", iface->name,
+		    hwaddr_ntoa(iface->hwaddr, iface->hwlen));
 }
 
 int
 select_profile(struct interface *iface, const char *profile)
 {
 	struct if_options *ifo;
+	int ret;
 
+	ret = 0;
 	ifo = read_config(cffile, iface->name, iface->ssid, profile);
 	if (ifo == NULL) {
 		syslog(LOG_DEBUG, "%s: no profile %s", iface->name, profile);
-		return -1;
+		ret = -1;
+		goto exit;
 	}
 	if (profile != NULL) {
 		strlcpy(iface->state->profile, profile,
@@ -830,8 +855,11 @@ select_profile(struct interface *iface, const char *profile)
 		*iface->state->profile = '\0';
 	free_options(iface->state->options);
 	iface->state->options = ifo;
-	configure_interface1(iface);
-	return 0;
+
+exit:
+	if (profile)
+		configure_interface1(iface);
+	return ret;
 }
 
 static void
@@ -841,7 +869,6 @@ start_fallback(void *arg)
 
 	iface = (struct interface *)arg;
 	select_profile(iface, iface->state->options->fallback);
-	configure_interface1(iface);
 	start_interface(iface);
 }
 
@@ -853,8 +880,8 @@ configure_interface(struct interface *iface, int argc, char **argv)
 	configure_interface1(iface);
 }
 
-static void
-handle_carrier(const char *ifname)
+void
+handle_carrier(int action, int flags, const char *ifname)
 {
 	struct interface *iface;
 	int carrier;
@@ -864,20 +891,36 @@ handle_carrier(const char *ifname)
 	for (iface = ifaces; iface; iface = iface->next)
 		if (strcmp(iface->name, ifname) == 0)
 			break;
-	if (!iface || !(iface->state->options->options & DHCPCD_LINK))
+	if (!iface) {
+		if (options & DHCPCD_LINK)
+			handle_interface(1, ifname);
 		return;
-	carrier = carrier_status(iface);
+	}
+	if (!(iface->state->options->options & DHCPCD_LINK))
+		return;
+
+	if (action) {
+		carrier = action == 1 ? 1 : 0;
+		iface->flags = flags;
+	} else
+		carrier = carrier_status(iface);
+
 	if (carrier == -1)
 		syslog(LOG_ERR, "%s: carrier_status: %m", ifname);
-	else if (carrier == 0 || !(iface->flags & IFF_RUNNING)) {
+	else if (carrier == 0 || ~iface->flags & IFF_UP) {
 		if (iface->carrier != LINK_DOWN) {
 			iface->carrier = LINK_DOWN;
 			syslog(LOG_INFO, "%s: carrier lost", iface->name);
 			close_sockets(iface);
 			delete_timeouts(iface, start_expire, NULL);
-			drop_config(iface, "NOCARRIER");
+			if (iface->ras) {
+				ipv6rs_free(iface);
+				iface->ras = NULL;
+				run_script_reason(iface, "ROUTERADVERT");
+			}
+			drop_dhcp(iface, "NOCARRIER");
 		}
-	} else if (carrier == 1 && (iface->flags & IFF_RUNNING)) {
+	} else if (carrier == 1 && !(~iface->flags & IFF_UP)) {
 		if (iface->carrier != LINK_UP) {
 			iface->carrier = LINK_UP;
 			syslog(LOG_INFO, "%s: carrier acquired", iface->name);
@@ -897,21 +940,37 @@ start_discover(void *arg)
 {
 	struct interface *iface = arg;
 	struct if_options *ifo = iface->state->options;
+	int timeout = ifo->timeout;
+
+	/* If we're rebooting and we're not daemonised then we need
+	 * to shorten the normal timeout to ensure we try correctly
+	 * for a fallback or IPv4LL address. */
+	if (iface->state->state == DHS_REBOOT &&
+	    !(options & DHCPCD_DAEMONISED))
+	{
+		timeout -= ifo->reboot;
+		if (timeout <= 0)
+			timeout = 2;
+	}
 
 	iface->state->state = DHS_DISCOVER;
 	iface->state->xid = dhcp_xid(iface);
 	delete_timeout(NULL, iface);
 	if (ifo->fallback)
-		add_timeout_sec(ifo->timeout, start_fallback, iface);
+		add_timeout_sec(timeout, start_fallback, iface);
 	else if (ifo->options & DHCPCD_IPV4LL &&
 	    !IN_LINKLOCAL(htonl(iface->addr.s_addr)))
 	{
 		if (IN_LINKLOCAL(htonl(iface->state->fail.s_addr)))
 			add_timeout_sec(RATE_LIMIT_INTERVAL, start_ipv4ll, iface);
 		else
-			add_timeout_sec(ifo->timeout, start_ipv4ll, iface);
+			add_timeout_sec(timeout, start_ipv4ll, iface);
 	}
-	syslog(LOG_INFO, "%s: broadcasting for a lease", iface->name);
+	if (ifo->options & DHCPCD_REQUEST)
+		syslog(LOG_INFO, "%s: broadcasting for a lease (requesting %s)",
+		    iface->name, inet_ntoa(ifo->req_addr));
+	else
+		syslog(LOG_INFO, "%s: broadcasting for a lease", iface->name);
 	send_discover(iface);
 }
 
@@ -1093,19 +1152,19 @@ start_interface(void *arg)
 	uint32_t l;
 	int nolease;
 
-	handle_carrier(iface->name);
+	handle_carrier(0, 0, iface->name);
 	if (iface->carrier == LINK_DOWN) {
 		syslog(LOG_INFO, "%s: waiting for carrier", iface->name);
 		return;
 	}
 
-	/* We don't want to read the old lease if we NAK an old test */
-	nolease = iface->state->offer && options & DHCPCD_TEST;
-
 	iface->start_uptime = uptime();
 	free(iface->state->offer);
 	iface->state->offer = NULL;
 
+	if (options & DHCPCD_IPV6RS && ifo->options & DHCPCD_IPV6RS)
+		ipv6rs_start(iface);
+
 	if (iface->state->arping_index < ifo->arping_len) {
 		start_arping(iface);
 		return;
@@ -1121,26 +1180,14 @@ start_interface(void *arg)
 	if (iface->hwlen == 0 && ifo->clientid[0] == '\0') {
 		syslog(LOG_WARNING, "%s: needs a clientid to configure",
 		    iface->name);
-		drop_config(iface, "FAIL");
+		drop_dhcp(iface, "FAIL");
 		close_sockets(iface);
 		delete_timeout(NULL, iface);
 		return;
 	}
-	if (ifo->req_addr.s_addr) {
-		iface->state->offer =
-		    dhcp_message_new(&ifo->req_addr, &ifo->req_mask);
-		if (ifo->options & DHCPCD_REQUEST)
-			ifo->req_addr.s_addr = 0;
-		else {
-			iface->state->reason = "STATIC";
-			iface->state->new = iface->state->offer;
-			iface->state->offer = NULL;
-			get_lease(&iface->state->lease, iface->state->new);
-			configure(iface);
-			start_inform(iface);
-			return;
-		}
-	} else if (!nolease)
+	/* We don't want to read the old lease if we NAK an old test */
+	nolease = iface->state->offer && options & DHCPCD_TEST;
+	if (!nolease)
 		iface->state->offer = read_lease(iface);
 	if (iface->state->offer) {
 		get_lease(&iface->state->lease, iface->state->offer);
@@ -1199,6 +1246,10 @@ init_state(struct interface *iface, int argc, char **argv)
 	configure_interface(iface, argc, argv);
 	if (!(options & DHCPCD_TEST))
 		run_script(iface);
+	/* We need to drop the leasefile so that start_interface
+	 * doesn't load it. */	
+	if (ifs->options->options & DHCPCD_REQUEST)
+		unlink(iface->leasefile);
 
 	if (ifs->options->options & DHCPCD_LINK) {
 		switch (carrier_status(iface)) {
@@ -1232,9 +1283,6 @@ handle_interface(int action, const char *ifname)
 		if (ifp != NULL)
 			stop_interface(ifp);
 		return;
-	} else if (action == 0) {
-		handle_carrier(ifname);
-		return;
 	}
 
 	/* If running off an interface list, check it's in it. */
@@ -1273,6 +1321,43 @@ handle_interface(int action, const char *ifname)
 	}
 }
 
+#ifdef RTM_CHGADDR
+void
+handle_hwaddr(const char *ifname, unsigned char *hwaddr, size_t hwlen)
+{
+	struct interface *ifp;
+	struct if_options *ifo;
+
+	for (ifp = ifaces; ifp; ifp = ifp->next)
+		if (strcmp(ifp->name, ifname) == 0 && ifp->hwlen <= hwlen) {
+			ifo = ifp->state->options;
+			if (!(ifo->options &
+			    (DHCPCD_INFORM | DHCPCD_STATIC | DHCPCD_CLIENTID))
+	    		    && ifp->state->new != NULL &&
+			    ifp->state->new->cookie == htonl(MAGIC_COOKIE))
+			{
+				syslog(LOG_INFO,
+				    "%s: expiring for new hardware address",
+				    ifp->name);
+				drop_dhcp(ifp, "EXPIRE");
+			}
+			memcpy(ifp->hwaddr, hwaddr, hwlen);
+			ifp->hwlen = hwlen;
+			if (!(ifo->options &
+			    (DHCPCD_INFORM | DHCPCD_STATIC | DHCPCD_CLIENTID)))
+			{
+				syslog(LOG_DEBUG, "%s: using hwaddr %s",
+				    ifp->name,
+		    		    hwaddr_ntoa(ifp->hwaddr, ifp->hwlen));
+				ifp->state->interval = 0;
+				ifp->state->nakoff = 1;
+				start_interface(ifp);
+			}
+		}
+	free(hwaddr);
+}
+#endif
+
 void
 handle_ifa(int type, const char *ifname,
     struct in_addr *addr, struct in_addr *net, struct in_addr *dst)
@@ -1288,43 +1373,44 @@ handle_ifa(int type, const char *ifname,
 			break;
 	if (ifp == NULL)
 		return;
+
+	if (type == RTM_DELADDR) {
+		if (ifp->state->new &&
+		    ifp->state->new->yiaddr == addr->s_addr)
+			syslog(LOG_INFO, "%s: removing IP address %s/%d",
+			    ifp->name, inet_ntoa(ifp->state->lease.addr),
+			    inet_ntocidr(ifp->state->lease.net));
+		return;
+	}
+
+	if (type != RTM_NEWADDR)
+		return;
+
 	ifo = ifp->state->options;
 	if ((ifo->options & (DHCPCD_INFORM | DHCPCD_STATIC)) == 0 ||
 	    ifo->req_addr.s_addr != INADDR_ANY)
 		return;
-	
-	switch (type) {
-	case RTM_DELADDR:
-		if (ifp->state->new &&
-		    ifp->state->new->yiaddr == addr->s_addr)
-			drop_config(ifp, "EXPIRE");
-		break;
-	case RTM_NEWADDR:
-		free(ifp->state->old);
-		ifp->state->old = ifp->state->new;
-		ifp->state->new = dhcp_message_new(addr, net);
-		ifp->dst.s_addr = dst ? dst->s_addr : INADDR_ANY;
-		if (dst) {
-			for (i = 1; i < 255; i++)
-				if (i != DHO_ROUTER &&
-				    has_option_mask(ifo->dstmask, i))
-					dhcp_message_add_addr(
-						ifp->state->new,
-						i, *dst);
-		}
-		ifp->state->reason = "STATIC";
-		build_routes();
-		run_script(ifp);
-		if (ifo->options & DHCPCD_INFORM) {
-			ifp->state->state = DHS_INFORM;
-			ifp->state->xid = dhcp_xid(ifp);
-			ifp->state->lease.server.s_addr =
-			    dst ? dst->s_addr : INADDR_ANY;
-			ifp->addr = *addr;
-			ifp->net = *net;
-			send_inform(ifp);
-		}
-		break;
+
+	free(ifp->state->old);
+	ifp->state->old = ifp->state->new;
+	ifp->state->new = dhcp_message_new(addr, net);
+	ifp->dst.s_addr = dst ? dst->s_addr : INADDR_ANY;
+	if (dst) {
+		for (i = 1; i < 255; i++)
+			if (i != DHO_ROUTER && has_option_mask(ifo->dstmask,i))
+				dhcp_message_add_addr(ifp->state->new, i, *dst);
+	}
+	ifp->state->reason = "STATIC";
+	build_routes();
+	run_script(ifp);
+	if (ifo->options & DHCPCD_INFORM) {
+		ifp->state->state = DHS_INFORM;
+		ifp->state->xid = dhcp_xid(ifp);
+		ifp->state->lease.server.s_addr =
+		    dst ? dst->s_addr : INADDR_ANY;
+		ifp->addr = *addr;
+		ifp->net = *net;
+		send_inform(ifp);
 	}
 }
 
@@ -1352,7 +1438,7 @@ if_reboot(struct interface *iface, int argc, char **argv)
 	    (opt & (DHCPCD_INFORM | DHCPCD_STATIC) &&
 		!(ifo->options & (DHCPCD_INFORM | DHCPCD_STATIC))))
 	{
-		drop_config(iface, "EXPIRE");
+		drop_dhcp(iface, "EXPIRE");
 	} else {
 		free(iface->state->offer);
 		iface->state->offer = NULL;
@@ -1369,25 +1455,6 @@ reconf_reboot(int action, int argc, char **argv, int oi)
 	if (ifs == NULL)
 		return;
 
-	/* Remove any old interfaces */
-	if (ifaces) {
-		for (ifl = NULL; ifl != ifaces;) {
-			/* Work our way backwards */
-			for (ifp = ifaces; ifp; ifp = ifp->next)
-				if (ifp->next == ifl) {
-					ifl = ifp;
-					break;
-				}
-			for (ifn = ifs; ifn; ifn = ifn->next)
-				if (strcmp(ifn->name, ifp->name) == 0)
-					break;
-			if (ifn == NULL) {
-				ifl = ifp->next;
-				stop_interface(ifp);
-			}
-		}
-	}
-	
 	for (ifp = ifs; ifp && (ift = ifp->next, 1); ifp = ift) {
 		ifl = NULL;
 		for (ifn = ifaces; ifn; ifn = ifn->next) {
@@ -1478,7 +1545,7 @@ handle_signal(_unused void *arg)
 	if (options & DHCPCD_TEST)
 		exit(EXIT_FAILURE);
 
-	/* As drop_config could re-arrange the order, we do it like this. */
+	/* As drop_dhcp could re-arrange the order, we do it like this. */
 	for (;;) {
 		/* Be sane and drop the last config first */
 		ifl = NULL;
@@ -1536,8 +1603,11 @@ handle_args(struct fd_list *fd, int argc, char **argv)
 		} else if (strcmp(*argv, "--getinterfaces") == 0) {
 			len = 0;
 			if (argc == 1) {
-				for (ifp = ifaces; ifp; ifp = ifp->next)
+				for (ifp = ifaces; ifp; ifp = ifp->next) {
 					len++;
+					if (ifp->ras)
+						len++;
+				}
 				len = write(fd->fd, &len, sizeof(len));
 				if (len != sizeof(len))
 					return -1;
@@ -1548,8 +1618,11 @@ handle_args(struct fd_list *fd, int argc, char **argv)
 			opt = 0;
 			while (argv[++opt] != NULL) {
 				for (ifp = ifaces; ifp; ifp = ifp->next)
-					if (strcmp(argv[opt], ifp->name) == 0)
+					if (strcmp(argv[opt], ifp->name) == 0) {
 						len++;
+						if (ifp->ras)
+							len++;
+					}
 			}
 			len = write(fd->fd, &len, sizeof(len));
 			if (len != sizeof(len))
@@ -1640,7 +1713,8 @@ open_sockets(struct interface *iface)
 	if (iface->udp_fd == -1 &&
 	    iface->addr.s_addr != 0 &&
 	    iface->state->new != NULL &&
-	    iface->state->new->cookie == htonl(MAGIC_COOKIE))
+	    (iface->state->new->cookie == htonl(MAGIC_COOKIE) ||
+	    iface->state->options->options & DHCPCD_INFORM))
 	{
 		if (open_udp_socket(iface) == -1 && errno != EADDRINUSE)
 			syslog(LOG_ERR, "%s: open_udp_socket: %m", iface->name);
@@ -1839,7 +1913,7 @@ main(int argc, char **argv)
 			if (sig != SIGALRM)
 				exit(EXIT_FAILURE);
 		} else {
-			if (sig == SIGALRM)
+			if (sig == SIGALRM || sig == SIGUSR1)
 				exit(EXIT_SUCCESS);
 			/* Spin until it exits */
 			syslog(LOG_INFO, "waiting for pid %d to exit", pid);
@@ -1866,29 +1940,25 @@ main(int argc, char **argv)
 		}
 
 		/* Ensure we have the needed directories */
-		if (mkdir(RUNDIR, 0755) == -1 && errno != EEXIST) {
+		if (mkdir(RUNDIR, 0755) == -1 && errno != EEXIST)
 			syslog(LOG_ERR, "mkdir `%s': %m", RUNDIR);
-			exit(EXIT_FAILURE);
-		}
-		if (mkdir(DBDIR, 0755) == -1 && errno != EEXIST) {
+		if (mkdir(DBDIR, 0755) == -1 && errno != EEXIST)
 			syslog(LOG_ERR, "mkdir `%s': %m", DBDIR);
-			exit(EXIT_FAILURE);
-		}
 
 		pidfd = open(pidfile, O_WRONLY | O_CREAT | O_NONBLOCK, 0664);
-		if (pidfd == -1) {
+		if (pidfd == -1)
 			syslog(LOG_ERR, "open `%s': %m", pidfile);
-			exit(EXIT_FAILURE);
+		else {
+			/* Lock the file so that only one instance of dhcpcd
+			 * runs on an interface */
+			if (flock(pidfd, LOCK_EX | LOCK_NB) == -1) {
+				syslog(LOG_ERR, "flock `%s': %m", pidfile);
+				exit(EXIT_FAILURE);
+			}
+			if (set_cloexec(pidfd) == -1)
+				exit(EXIT_FAILURE);
+			writepid(pidfd, getpid());
 		}
-		/* Lock the file so that only one instance of dhcpcd runs
-		 * on an interface */
-		if (flock(pidfd, LOCK_EX | LOCK_NB) == -1) {
-			syslog(LOG_ERR, "flock `%s': %m", pidfile);
-			exit(EXIT_FAILURE);
-		}
-		if (set_cloexec(pidfd) == -1)
-			exit(EXIT_FAILURE);
-		writepid(pidfd, getpid());
 	}
 
 	syslog(LOG_INFO, "version " VERSION " starting");
@@ -1900,10 +1970,8 @@ main(int argc, char **argv)
 	add_event(signal_fd, handle_signal, NULL);
 
 	if (options & DHCPCD_MASTER) {
-		if (start_control() == -1) {
+		if (start_control() == -1)
 			syslog(LOG_ERR, "start_control: %m");
-			exit(EXIT_FAILURE);
-		}
 	}
 
 	if (init_sockets() == -1) {
@@ -1918,6 +1986,26 @@ main(int argc, char **argv)
 			add_event(linkfd, handle_link, NULL);
 	}
 
+#if 0
+	if (options & DHCPCD_IPV6RS && disable_rtadv() == -1) {
+		syslog(LOG_ERR, "ipv6rs: %m");
+		options &= ~DHCPCD_IPV6RS;
+	}
+#endif
+
+	if (options & DHCPCD_IPV6RS && !check_ipv6())
+		options &= ~DHCPCD_IPV6RS;
+	if (options & DHCPCD_IPV6RS) {
+		ipv6rsfd = ipv6rs_open();
+		if (ipv6rsfd == -1) {
+			syslog(LOG_ERR, "ipv6rs: %m");
+			options &= ~DHCPCD_IPV6RS;
+		} else {
+			add_event(ipv6rsfd, ipv6rs_handledata, NULL);
+//			atexit(restore_rtadv);
+		}
+	}
+
 	ifc = argc - optind;
 	ifv = argv + optind;
 
@@ -1959,7 +2047,7 @@ main(int argc, char **argv)
 
 	if (!(options & DHCPCD_BACKGROUND)) {
 		/* If we don't have a carrier, we may have to wait for a second
-		 * before one becomes available if we brought an interface up. */
+		 * before one becomes available if we brought an interface up */
 		if (opt == 0 &&
 		    options & DHCPCD_LINK &&
 		    options & DHCPCD_WAITUP &&
@@ -1969,24 +2057,27 @@ main(int argc, char **argv)
 			ts.tv_nsec = 0;
 			nanosleep(&ts, NULL);
 			for (iface = ifaces; iface; iface = iface->next) {
-				handle_carrier(iface->name);
+				handle_carrier(0, 0, iface->name);
 				if (iface->carrier != LINK_DOWN) {
 					opt = 1;
 					break;
 				}
 			}
 		}
+		if (options & DHCPCD_MASTER)
+			i = if_options->timeout;
+		else
+			i = ifaces->state->options->timeout;
 		if (opt == 0 &&
 		    options & DHCPCD_LINK &&
 		    !(options & DHCPCD_WAITIP))
 		{
 			syslog(LOG_WARNING, "no interfaces have a carrier");
 			daemonise();
-		} else if (if_options->timeout > 0) {
+		} else if (i > 0) {
 			if (options & DHCPCD_IPV4LL)
 				options |= DHCPCD_TIMEOUT_IPV4LL;
-			add_timeout_sec(if_options->timeout,
-			    handle_exit_timeout, NULL);
+			add_timeout_sec(i, handle_exit_timeout, NULL);
 		}
 	}
 	free_options(if_options);
