@@ -1,6 +1,6 @@
 /* 
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2010 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2011 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -53,6 +53,7 @@
 #define O_ARPING	O_BASE + 1
 #define O_FALLBACK	O_BASE + 2
 #define O_DESTINATION	O_BASE + 3
+#define O_NOIPV6RS	O_BASE + 4
 
 const struct option cf_options[] = {
 	{"background",      no_argument,       NULL, 'b'},
@@ -103,6 +104,7 @@ const struct option cf_options[] = {
 	{"arping",          required_argument, NULL, O_ARPING},
 	{"destination",     required_argument, NULL, O_DESTINATION},
 	{"fallback",        required_argument, NULL, O_FALLBACK},
+	{"noipv6rs",        no_argument,       NULL, O_NOIPV6RS},
 	{NULL,              0,                 NULL, '\0'}
 };
 
@@ -420,14 +422,12 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 		ifo->options |= DHCPCD_QUIET;
 		break;
 	case 'r':
-		ifo->options |= DHCPCD_REQUEST;
 		if (parse_addr(&ifo->req_addr, NULL, arg) != 0)
 			return -1;
+		ifo->options |= DHCPCD_REQUEST;
 		ifo->req_mask.s_addr = 0;
 		break;
 	case 's':
-		ifo->options |= DHCPCD_INFORM | DHCPCD_PERSISTENT;
-		ifo->options &= ~(DHCPCD_ARP | DHCPCD_STATIC);
 		if (arg && *arg != '\0') {
 			if (parse_addr(&ifo->req_addr, &ifo->req_mask,
 				arg) != 0)
@@ -436,6 +436,8 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 			ifo->req_addr.s_addr = 0;
 			ifo->req_mask.s_addr = 0;
 		}
+		ifo->options |= DHCPCD_INFORM | DHCPCD_PERSISTENT;
+		ifo->options &= ~(DHCPCD_ARP | DHCPCD_STATIC);
 		break;
 	case 't':
 		ifo->timeout = atoint(arg);
@@ -625,11 +627,16 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 		}
 		p++;
 		if (strncmp(arg, "ip_address=", strlen("ip_address=")) == 0) {
-			if (parse_addr(&ifo->req_addr, &ifo->req_mask, p) != 0)
+			if (parse_addr(&ifo->req_addr,
+			    ifo->req_mask.s_addr == 0 ? &ifo->req_mask : NULL,
+			    p) != 0)
 				return -1;
 
 			ifo->options |= DHCPCD_STATIC;
 			ifo->options &= ~DHCPCD_INFORM;
+		} else if (strncmp(arg, "subnet_mask=", strlen("subnet_mask=")) == 0) {
+			if (parse_addr(&ifo->req_mask, NULL, p) != 0)
+				return -1;
 		} else if (strncmp(arg, "routes=", strlen("routes=")) == 0 ||
 		    strncmp(arg, "static_routes=", strlen("static_routes=")) == 0 ||
 		    strncmp(arg, "classless_static_routes=", strlen("classless_static_routes=")) == 0 ||
@@ -735,6 +742,9 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 		free(ifo->fallback);
 		ifo->fallback = xstrdup(arg);
 		break;
+	case O_NOIPV6RS:
+		ifo->options &=~ DHCPCD_IPV6RS;
+		break;
 	default:
 		return 0;
 	}
@@ -778,8 +788,8 @@ read_config(const char *file,
 
 	/* Seed our default options */
 	ifo = xzalloc(sizeof(*ifo));
-	ifo->options |= DHCPCD_GATEWAY | DHCPCD_DAEMONISE;
-	ifo->options |= DHCPCD_ARP | DHCPCD_IPV4LL | DHCPCD_LINK;
+	ifo->options |= DHCPCD_GATEWAY | DHCPCD_DAEMONISE | DHCPCD_LINK;
+	ifo->options |= DHCPCD_ARP | DHCPCD_IPV4LL | DHCPCD_IPV6RS;
 	ifo->timeout = DEFAULT_TIMEOUT;
 	ifo->reboot = DEFAULT_REBOOT;
 	ifo->metric = -1;
