@@ -1,6 +1,6 @@
 /* 
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2010 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2012 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -27,7 +27,9 @@
 
 #include <errno.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
+#include <syslog.h>
 
 #include "common.h"
 #include "platform.h"
@@ -85,17 +87,53 @@ hardware_platform(void)
 	if (fp == NULL)
 		return NULL;
 
+	p = NULL;
 	while ((buf = get_line(fp))) {
 		if (strncmp(buf, mproc, strlen(mproc)) == 0) {
 			p = strchr(buf, ':');
 			if (p != NULL && ++p != NULL) {
 				while (*p == ' ')
 					p++;
-				return p;
+				break;
 			}
 		}
 	}
+	fclose(fp);
 
-	errno = ESRCH;
-	return NULL;
+	if (p == NULL)
+		errno = ESRCH;
+	return p;
+}
+
+static int
+check_proc_int(const char *path)
+{
+	FILE *fp;
+	char *buf;
+
+	fp = fopen(path, "r");
+	if (fp == NULL)
+		return -1;
+	buf = get_line(fp);
+	fclose(fp);
+	if (buf == NULL)
+		return -1;
+	return atoi(buf);
+}
+
+int
+check_ipv6(void)
+{
+
+	if (check_proc_int("/proc/sys/net/ipv6/conf/all/accept_ra") != 1) {
+		syslog(LOG_WARNING,
+		    "Kernel is not configured to accept IPv6 RAs");
+		return 0;
+	}
+	if (check_proc_int("/proc/sys/net/ipv6/conf/all/forwarding") != 0) {
+		syslog(LOG_WARNING,
+		    "Kernel is configured as a router, not a host");
+		return 0;
+	}
+	return 1;
 }
